import { createRouter, createWebHistory } from "vue-router";
import NotFound from "@/views/sessions/404.vue";
import { useLargeSidebarStore } from "@/store/largeSidebarStore";

const routes = [
  {
    path: "/",
    name: "Home",
    component: () => import("../layout/index.vue"),
    meta: {
      title: "Home",
    },
    redirect: "/misbebes/listado",
    children: [
      {
        path: "/misbebes",
        redirect: "/misbebes/listado",
        name: "patients",
        component: () => import("../views/patients/index.vue"),
        meta: {
          title: "Mis pacientes",
        },
        children: [
          {
            path: "listado",
            name: "PatientsListPage",
            component: () => import("../views/patients/PatientsListPage.vue"),
            meta: {
              title: "Listado pacientes",
            },
          },
          {
            path: "agregar",
            name: "PatientAddPage",
            component: () => import("../views/patients/PatientAddPage.vue"),
            meta: {
              title: "Agregar paciente",
            },
          },
          {
            path: "editar/:id",
            name: "PatientEditPage",
            component: () => import("../views/patients/PatientEditPage.vue"),
            meta: {
              title: "Editar paciente",
            },
          },
          {
            path: "detalles/:id",
            name: "PatientDetailsPage",
            component: () => import("../views/patients/PatientDetailsPage.vue"),
            meta: {
              title: "Detalles paciente",
            },
          },
        ],
      },
      {
        path: "/consultas",
        name: "consultations",
        redirect: "/consultas/periodicas",
        component: () => import("../views/consultations/index.vue"),
        meta: {
          title: "Consultas médicas",
        },
        children: [
          {
            path: "listado",
            name: "ConsultationsListPage",
            component: () =>
              import("../views/consultations/ConsultationsListPage.vue"),
            meta: {
              title: "Consultas médicas",
            },
          },
          {
            path: "agregar",
            name: "ConsultationAddPage",
            component: () =>
              import("../views/consultations/ConsultationAddPage.vue"),
            meta: {
              title: "Agregar consulta",
            },
          },
          {
            path: "editar/:id",
            name: "ConsultationEditPage",
            component: () =>
              import("../views/consultations/ConsultationEditPage.vue"),
            meta: {
              title: "Editar consulta",
            },
          },
          {
            path: "detalles/:id",
            name: "ConsultationDetailsPage",
            component: () =>
              import("../views/consultations/ConsultationDetailsPage.vue"),
            meta: {
              title: "Detalles consulta",
            },
          },
        ],
      },
    ],
  },
  { path: "/:path(.*)", component: NotFound },
];

const router = createRouter({
  history: createWebHistory(),
  //scrollBehavior(to, from, savedPosition) {
  //  return { left: 0, top: 0 };
  //},
  routes,
});
router.beforeEach((to, from, next) => {
  const appTitle = import.meta.env.VITE_APP_TITLE;
  document.title = to.meta.title ? `${to.meta.title} - ${appTitle}` : appTitle;
  next();
});
router.afterEach(() => {
  const store = useLargeSidebarStore();

  const loader = document.getElementById("loading_wrap");
  if (loader) {
    loader.style.display = "none";
  }

  const sideNav = store.sidebarToggleProperties.isSideNavOpen;
  const secondarySideNav = store.sidebarToggleProperties.isSecondarySideNavOpen;

  if (window.innerWidth <= 1200) {
    if (sideNav) {
      store.toggleSidebarProperties();
    }
    if (secondarySideNav) {
      store.toggleSecondarySidebarProperties();
    }
  } else {
    if (secondarySideNav) {
      store.toggleSecondarySidebarProperties();
    }
  }
});

export default router;
