import {
  addDays,
  addMonths,
  addYears,
  formatDistance,
  isPast,
  isToday,
} from "date-fns";
import { es } from "date-fns/locale";

export default function formatPeriod(period) {
  // Establecer hoy como referencia
  let now = new Date();

  let match = /^P((-?\d+)Y)?((-?\d+)M)?((-?\d+)D)?$/.exec(period);

  if (!match) {
    throw new Error("Formato de período incorrecto");
  }

  let years = match[2] ? parseInt(match[2]) : 0;
  let months = match[4] ? parseInt(match[4]) : 0;
  let days = match[6] ? parseInt(match[6]) : 0;

  let date = addDays(addMonths(addYears(now, years), months), days);

  if (isToday(date)) {
    return "Hoy";
  } else if (isPast(date)) {
    return `Hace ${formatDistance(now, date, {
      addSuffix: false,
      locale: es,
    })}`;
  } else {
    return `Dentro de ${formatDistance(date, now, {
      addSuffix: false,
      locale: es,
    })}`;
  }
}
