package es.mdef.mipediatra.assemblers;

import es.mdef.mipediatra.controllers.ConsultationsController;
import es.mdef.mipediatra.controllers.MeasurementsController;
import es.mdef.mipediatra.entities.MeasurementEntity;
import es.mdef.mipediatra.models.measurement.MeasurementModel;
import es.mdef.mipediatra.models.measurement.MeasurementPostModel;
import es.mdef.mipediatra.models.measurement.MeasurementPutModel;
import es.mdef.mipediatra.models.measurement.MeasurementsListModel;
import es.mdef.mipediatra.services.AssemblerService;
import org.springframework.stereotype.Component;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@Component
public class MeasurementAssembler extends AssemblerService<MeasurementEntity, MeasurementModel, MeasurementPostModel, MeasurementPutModel, MeasurementsListModel> {
    public MeasurementAssembler() {
        super(() -> linkTo(methodOn(MeasurementsController.class).getAll()).withSelfRel());
    }

    @Override
    public MeasurementModel toModel(MeasurementEntity measurement) {
        MeasurementModel measurementModel = new MeasurementModel();
        measurementModel.setCranialDiameter(measurement.getCranialDiameter());
        measurementModel.setHeight(measurement.getHeight());
        measurementModel.setWeight(measurement.getWeight());

        return measurementModel;
    }

    @Override
    public MeasurementsListModel toListModel(MeasurementEntity measurement) {
        MeasurementsListModel measurementModel = new MeasurementsListModel();
        measurementModel.setCranialDiameter(measurement.getCranialDiameter());
        measurementModel.setHeight(measurement.getHeight());
        measurementModel.setWeight(measurement.getWeight());

        measurementModel.add(linkTo(methodOn(MeasurementsController.class).getById(measurement.getId())).withSelfRel())
                .add(linkTo(methodOn(ConsultationsController.class).getAll()).withRel("measurements"));
        return measurementModel;
    }

    @Override
    public MeasurementEntity toEntity(MeasurementPostModel model) {
        MeasurementEntity measurement = new MeasurementEntity();
        measurement.setHeight(model.getHeight());
        measurement.setWeight(model.getWeight());
        measurement.setCranialDiameter(model.getCranialDiameter());
        return measurement;
    }

    @Override
    public MeasurementEntity toEntity(MeasurementPutModel model) {
        return null;
    }

    @Override
    public MeasurementEntity update(MeasurementEntity entity, MeasurementPutModel model) {
        entity.setHeight(model.getHeight());
        entity.setWeight(model.getWeight());
        entity.setCranialDiameter(model.getCranialDiameter());
        return entity;
    }
}
