package es.mdef.mipediatra.assemblers;

import es.mdef.mipediatra.constants.PatientType;
import es.mdef.mipediatra.controllers.PatientsController;
import es.mdef.mipediatra.entities.AutonomousCommunityEntity;
import es.mdef.mipediatra.entities.FileEntity;
import es.mdef.mipediatra.entities.PatientEntity;
import es.mdef.mipediatra.entities.PersonEntity;
import es.mdef.mipediatra.models.patient.*;
import es.mdef.mipediatra.services.AssemblerService;
import org.springframework.stereotype.Component;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@Component
public class PatientAssembler extends AssemblerService<PatientEntity, PatientModel, PatientPostModel, PatientPutModel, PatientListModel> {

    private final AutonomousCommunityAssembler autonomousCommunityAssembler;
    private final FileAssembler fileAssembler;

    public PatientAssembler(AutonomousCommunityAssembler autonomousCommunityAssembler, FileAssembler fileAssembler) {
        super(() -> linkTo(methodOn(PatientsController.class).getAll()).withSelfRel());
        this.autonomousCommunityAssembler = autonomousCommunityAssembler;
        this.fileAssembler = fileAssembler;
    }

    @Override
    public PatientModel toModel(PatientEntity patient) {
        PatientModel model = new PatientModel();

        if (patient instanceof PersonEntity personEntity) {
            PersonModel personModel = new PersonModel();
            personModel.setName(personEntity.getName());
            personModel.setLastName(personEntity.getLastName());
            personModel.setBirthWeight(personEntity.getBirthWeight());
            personModel.setGender(personEntity.getGender());
            personModel.setBloodType(personEntity.getBloodType());
            personModel.setBirthdate(personEntity.getBirthdate());
            personModel.setAutonomousCommunity(autonomousCommunityAssembler
                    .toModel((AutonomousCommunityEntity) personEntity.getAutonomousCommunity()));
            //TODO: Add link a vacunas
            model = personModel;
        }

        model.setComments(patient.getComments());
        if (patient.getPhoto() != null) {
            model.setPhoto(fileAssembler.toModel((FileEntity) patient.getPhoto()));
        }
        //TODO: Add link a consultas
        model.add(linkTo(methodOn(PatientsController.class).getById(patient.getId())).withSelfRel());
        return model;
    }

    @Override
    public PatientListModel toListModel(PatientEntity patient) {
        PatientListModel model = new PatientListModel();

        if (patient instanceof PersonEntity personEntity) {
            PersonListModel personModel = new PersonListModel();
            personModel.setName(personEntity.getName());
            personModel.setLastName(personEntity.getLastName());
            personModel.setGender(personEntity.getGender());
            personModel.setBirthdate(personEntity.getBirthdate());
            //TODO: Add link a vacunas
            model = personModel;
        }

        model.setComments(patient.getComments());

        model.setComments(patient.getComments());
        if (patient.getPhoto() != null) {
            model.setPhoto(fileAssembler.toModel((FileEntity) patient.getPhoto()));
        }
        model.add(linkTo(methodOn(PatientsController.class).getById(patient.getId())).withSelfRel());
        return model;
    }

    @Override
    public PatientEntity toEntity(PatientPostModel model) {
        PatientEntity entity = null;

        if (model.getType() == PatientType.PERSON) {
            PersonEntity personEntity = new PersonEntity();
            personEntity.setName(model.getName());
            personEntity.setLastName(model.getLastName());
            personEntity.setAutonomousCommunity(model.getAutonomousCommunity());
            personEntity.setBirthWeight(model.getBirthWeight());
            personEntity.setGender(model.getGender());
            personEntity.setBloodType(model.getBloodType());
            personEntity.setBirthdate(model.getBirthdate());
            entity = personEntity;
        }

        if (entity != null) {
            entity.setPhoto(model.getPhoto());
            entity.setComments(model.getComments());
        }

        return entity;
    }

    @Override
    public PatientEntity toEntity(PatientPutModel model) {
        PatientEntity entity = null;

        if (model.getType() == PatientType.PERSON) {
            PersonEntity personEntity = new PersonEntity();
            personEntity.setName(model.getName());
            personEntity.setLastName(model.getLastName());
            personEntity.setAutonomousCommunity(model.getAutonomousCommunity());
            personEntity.setBirthWeight(model.getBirthWeight());
            personEntity.setGender(model.getGender());
            personEntity.setBloodType(model.getBloodType());
            personEntity.setBirthdate(model.getBirthdate());
            entity = personEntity;
        }

        if (entity != null) {
            entity.setPhoto(model.getPhoto());
            entity.setComments(model.getComments());
        }

        return entity;
    }

    @Override
    public PatientEntity update(PatientEntity patient, PatientPutModel model) {
        if (patient != null && model.getType() == PatientType.PERSON) {
            PersonEntity person = (PersonEntity) patient;
            person.setName(model.getName());
            person.setLastName(model.getLastName());
            person.setAutonomousCommunity(model.getAutonomousCommunity());
            person.setBirthWeight(model.getBirthWeight());
            person.setGender(model.getGender());
            person.setBloodType(model.getBloodType());
            person.setBirthdate(model.getBirthdate());
        }

        if (patient != null) {
            patient.setPhoto(model.getPhoto());
            patient.setComments(model.getComments());
        }
        return patient;
    }
}
