package es.mdef.mipediatra.assemblers;

import es.mdef.mipediatra.controllers.PatientsController;
import es.mdef.mipediatra.controllers.VaccinesController;
import es.mdef.mipediatra.controllers.VaccinesPersonsController;
import es.mdef.mipediatra.entities.FileEntity;
import es.mdef.mipediatra.entities.PersonEntity;
import es.mdef.mipediatra.entities.VaccineEntity;
import es.mdef.mipediatra.entities.VaccinesPersonsEntity;
import es.mdef.mipediatra.models.vaccine.VaccinesPersonsListModel;
import es.mdef.mipediatra.models.vaccine.VaccinesPersonsModel;
import es.mdef.mipediatra.models.vaccine.VaccinesPersonsPostModel;
import es.mdef.mipediatra.models.vaccine.VaccinesPersonsPutModel;
import es.mdef.mipediatra.services.AssemblerService;
import es.mdef.mipediatralib.interfaces.IPerson;
import org.springframework.stereotype.Component;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@Component
public class VaccinesPersonsAssembler extends AssemblerService<VaccinesPersonsEntity, VaccinesPersonsModel, VaccinesPersonsPostModel, VaccinesPersonsPutModel, VaccinesPersonsListModel> {
    private final FileAssembler fileAssembler;

    public VaccinesPersonsAssembler(FileAssembler fileAssembler) {
        super(() -> linkTo(methodOn(VaccinesPersonsController.class).getAll()).withSelfRel());
        this.fileAssembler = fileAssembler;
    }

    public VaccinesPersonsModel toModel(VaccinesPersonsEntity entity) {
        VaccinesPersonsModel model = new VaccinesPersonsModel();

        String name = entity.getVaccine().getVaccineDetails().getName();
        String shortName = entity.getVaccine().getVaccineDetails().getShortName();
        int dose = entity.getVaccine().getDose();
        String doseToDisplay = dose != 0 ? dose + "º dosis" : "Anual";

        String finalName = name + " (" + shortName + ")" + " - " + doseToDisplay;

        model.setName(finalName);
        model.setDate(entity.getDate());
        model.setExpectedDate(entity.getExpectedDate());
        model.setPhoto(fileAssembler.toModel((FileEntity) entity.getPhoto()));
        model.setReaction(entity.getReaction());
        model.setHasBeenAdministered(entity.isHasBeenAdministered());

        model.add(linkTo(methodOn(PatientsController.class).getById(((PersonEntity) entity.getPerson()).getId())).withRel("patient"));
        model.add(linkTo(methodOn(VaccinesController.class).getById(((VaccineEntity) entity.getVaccine()).getId())).withRel("vaccine"));
        model.add(linkTo(methodOn(VaccinesPersonsController.class).getById(entity.getId())).withSelfRel());
        return model;
    }

    @Override
    public VaccinesPersonsListModel toListModel(VaccinesPersonsEntity entity) {
        VaccinesPersonsListModel model = new VaccinesPersonsListModel();

        String name = entity.getVaccine().getVaccineDetails().getName();
        String shortName = entity.getVaccine().getVaccineDetails().getShortName();
        int dose = entity.getVaccine().getDose();
        String doseToDisplay = dose != 0 ? dose + "º dosis" : "Anual";

        String finalName = name + " (" + shortName + ")" + " - " + doseToDisplay;

        model.setName(finalName);
        model.setDate(entity.getDate());
        model.setExpectedDate(entity.getExpectedDate());
        model.setPhoto(fileAssembler.toModel((FileEntity) entity.getPhoto()));
        model.setReaction(entity.getReaction());
        model.setHasBeenAdministered(entity.isHasBeenAdministered());

        model.add(linkTo(methodOn(PatientsController.class).getById(((PersonEntity) entity.getPerson()).getId())).withRel("patient"));
        model.add(linkTo(methodOn(VaccinesController.class).getById(((VaccineEntity) entity.getVaccine()).getId())).withRel("vaccine"));
        model.add(linkTo(methodOn(VaccinesPersonsController.class).getById(entity.getId())).withSelfRel());
        return model;
    }

    public VaccinesPersonsEntity toEntity(VaccinesPersonsPostModel model) {
        VaccinesPersonsEntity entity = new VaccinesPersonsEntity();
        entity.setVaccine(model.getVaccine());
        entity.setPerson((IPerson) model.getPerson());
        entity.setDate(model.getDate());
        entity.setExpectedDate(model.getExpectedDate());
        entity.setPhoto(model.getPhoto());
        entity.setReaction(model.getReaction());
        entity.setHasBeenAdministered(model.isHasBeenAdministered());
        return entity;
    }

    @Override
    public VaccinesPersonsEntity toEntity(VaccinesPersonsPutModel model) {
        return null;
    }

    @Override
    public VaccinesPersonsEntity update(VaccinesPersonsEntity entity, VaccinesPersonsPutModel model) {
        entity.setVaccine(model.getVaccine());
        entity.setDate(model.getDate());
        entity.setExpectedDate(model.getExpectedDate());
        entity.setPhoto(model.getPhoto());
        entity.setReaction(model.getReaction());
        entity.setHasBeenAdministered(model.isHasBeenAdministered());
        return entity;
    }
}
