package es.mdef.mipediatra.controllers;

import es.mdef.mipediatra.entities.ConsultationEntity;
import es.mdef.mipediatra.models.consultation.*;
import es.mdef.mipediatra.models.file.FileListModel;
import es.mdef.mipediatra.models.symptom.SymptomsListModel;
import es.mdef.mipediatra.services.ControllerService;
import es.mdef.mipediatra.services.CrudService;
import es.mdef.mipediatra.services.impl.ConsultationService;
import lombok.extern.log4j.Log4j2;
import org.springframework.hateoas.CollectionModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@CrossOrigin(origins = "*")
@RequestMapping("/consultations")
@Log4j2
public class ConsultationsController implements ControllerService<ConsultationEntity, ConsultationModel, ConsultationPostModel, ConsultationPutModel, ConsultationListModel> {
    private final ConsultationService consultationService;

    public ConsultationsController(ConsultationService consultationService) {
        this.consultationService = consultationService;
    }

    @GetMapping("{id}/symptoms")
    public ResponseEntity<CollectionModel<SymptomsListModel>> symptoms(@PathVariable long id) {
        return ResponseEntity.ok(consultationService.getSymptoms(id));
    }

    @GetMapping("{id}/files")
    public ResponseEntity<CollectionModel<FileListModel>> files(@PathVariable long id) {
        return ResponseEntity.ok(consultationService.getFiles(id));
    }

    @PostMapping("{id}/files")
    public ResponseEntity<Void> files(@PathVariable long id, @RequestBody ConsultationAddFileModel model) {
        consultationService.addFile(id, model.getFile());
        return ResponseEntity.ok().build();
    }

    @DeleteMapping("{id}/files/{fileId}")
    public ResponseEntity<Void> removeFile(@PathVariable long id, @PathVariable long fileId) {
        consultationService.removeFile(id, fileId);
        return ResponseEntity.ok().build();
    }

    @Override
    public CrudService<ConsultationEntity, ConsultationModel, ConsultationPostModel, ConsultationPutModel, ConsultationListModel> getService() {
        return consultationService;
    }
}
