package es.mdef.mipediatra.controllers;

import es.mdef.mipediatra.entities.PatientEntity;
import es.mdef.mipediatra.models.patient.PatientListModel;
import es.mdef.mipediatra.models.patient.PatientModel;
import es.mdef.mipediatra.models.patient.PatientPostModel;
import es.mdef.mipediatra.models.patient.PatientPutModel;
import es.mdef.mipediatra.models.symptom.SymptomsListModel;
import es.mdef.mipediatra.models.vaccine.VaccinesPersonsListModel;
import es.mdef.mipediatra.services.ControllerService;
import es.mdef.mipediatra.services.CrudService;
import es.mdef.mipediatra.services.impl.PatientsService;
import es.mdef.mipediatra.services.impl.VaccinesPersonsService;
import org.springframework.hateoas.CollectionModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@CrossOrigin(origins = "*")
@RequestMapping("/patients")
public class PatientsController implements ControllerService<PatientEntity, PatientModel, PatientPostModel, PatientPutModel, PatientListModel> {
    private final PatientsService patientsService;

    public PatientsController(PatientsService patientsService, VaccinesPersonsService vaccinesPersonsService) {
        this.patientsService = patientsService;
    }

    @GetMapping("{id}/vaccines")
    public ResponseEntity<CollectionModel<VaccinesPersonsListModel>> getVaccines(@PathVariable Long id) {
        return ResponseEntity.ok(patientsService.getVaccines(id));
    }

    @GetMapping("{id}/symptoms")
    public ResponseEntity<CollectionModel<SymptomsListModel>> getSymptoms(@PathVariable Long id, @RequestParam(required = false, defaultValue = "5") int num) {
        return ResponseEntity.ok(patientsService.getSymptoms(id, num));
    }

    @Override
    public CrudService<PatientEntity, PatientModel, PatientPostModel, PatientPutModel, PatientListModel> getService() {
        return patientsService;
    }
}