package es.mdef.mipediatra.data.impl;

import es.mdef.mipediatra.data.Seeder;
import es.mdef.mipediatra.entities.EmergencyEntity;
import es.mdef.mipediatra.entities.MeasurementEntity;
import es.mdef.mipediatra.entities.PersonEntity;
import es.mdef.mipediatra.entities.RegularVisitEntity;
import es.mdef.mipediatra.repositories.AutonomousCommunityRepository;
import es.mdef.mipediatra.repositories.ConsultationRepository;
import es.mdef.mipediatra.repositories.PatientRepository;
import es.mdef.mipediatra.repositories.SymptomRepository;
import es.mdef.mipediatralib.constants.BloodType;
import es.mdef.mipediatralib.constants.Gender;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;

@Service
public class PatientSeeder implements Seeder {
    private final PatientRepository patientRepository;
    private final AutonomousCommunityRepository autonomousCommunityRepository;
    private final ConsultationRepository consultationRepository;
    private final SymptomRepository symptomRepository;

    public PatientSeeder(PatientRepository patientRepository, AutonomousCommunityRepository autonomousCommunityRepository, ConsultationRepository consultationRepository, SymptomRepository symptomRepository) {
        this.patientRepository = patientRepository;
        this.autonomousCommunityRepository = autonomousCommunityRepository;
        this.consultationRepository = consultationRepository;
        this.symptomRepository = symptomRepository;
    }

    @Override
    public void seed() {
        if (patientRepository.findAll().isEmpty()) {
            PersonEntity person1 = new PersonEntity();
            person1.setName("Javier");
            person1.setLastName("Tomás Acín");
            person1.setGender(Gender.HOMBRE);
            person1.setBirthdate(ZonedDateTime.of(LocalDateTime.of(1988, 11, 9, 13, 0), ZoneId.of("Europe/Madrid")));
            person1.setBirthWeight(3.4);
            person1.setComments("Hay que vigilarlo");
            person1.setAutonomousCommunity(autonomousCommunityRepository.findByName("Aragón"));
            person1.setBloodType(BloodType.A_POSITIVE);
            patientRepository.save(person1);

            MeasurementEntity measurement = new MeasurementEntity();
            measurement.setCranialDiameter(35);
            measurement.setHeight(35);
            measurement.setWeight(15);

            RegularVisitEntity regularVisit = new RegularVisitEntity();
            regularVisit.setPatient(person1);
            regularVisit.setComments("El médico dijo que estaba evolucionando bien.");
            regularVisit.setMeasurement(measurement);
            regularVisit.setCenter("Hospital Miguel Servet");
            regularVisit.setDate(ZonedDateTime.of(LocalDateTime.of(2023, 2, 3, 16, 30), ZoneId.of("Europe/Madrid")));
            regularVisit.setSpecialist("Dr. Sánchez");
            regularVisit.setPatient(person1);
            consultationRepository.save(regularVisit);

            EmergencyEntity emergency = new EmergencyEntity();
            emergency.setPatient(person1);
            emergency.setComments("Tosía mucho.");
            emergency.setCenter("Hospital San Jorge");
            emergency.setDate(ZonedDateTime.of(LocalDateTime.of(2023, 4, 15, 19, 15), ZoneId.of("Europe/Madrid")));
            emergency.setSpecialist("Dr. Valerio");
            emergency.setDiagnosis("Anginas");
            emergency.setTreatment("Antibiótico");

            emergency.getSymptoms().add(symptomRepository.findByName("Tos"));
            emergency.getSymptoms().add(symptomRepository.findByName("Diarrea"));
            consultationRepository.save(emergency);
            patientRepository.save(person1);

            PersonEntity person2 = new PersonEntity();
            person2.setName("Isabel");
            person2.setLastName("Tomás Acín");
            person2.setGender(Gender.MUJER);
            person2.setBirthdate(ZonedDateTime.of(LocalDateTime.of(1989, 6, 1, 15, 45), ZoneId.of("Europe/Madrid")));
            person2.setBirthWeight(2.8);
            person2.setComments("Crece adecuadamente");
            person2.setAutonomousCommunity(autonomousCommunityRepository.findByName("Aragón"));
            person2.setBloodType(BloodType.AB_POSITIVE);
            patientRepository.save(person2);
        }
    }
}
