package es.mdef.mipediatra.data.impl;

import es.mdef.mipediatra.data.Seeder;
import es.mdef.mipediatra.entities.AutonomousCommunityEntity;
import es.mdef.mipediatra.entities.VaccineDetailsEntity;
import es.mdef.mipediatra.entities.VaccineDosePeriodEntity;
import es.mdef.mipediatra.entities.VaccineEntity;
import es.mdef.mipediatra.repositories.AutonomousCommunityRepository;
import es.mdef.mipediatra.repositories.VaccineDetailsRepository;
import es.mdef.mipediatra.repositories.VaccineDosePeriodRepository;
import es.mdef.mipediatra.repositories.VaccineRepository;
import org.springframework.data.util.Pair;
import org.springframework.stereotype.Service;

import java.time.Period;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

@Service
public class VaccinesSeeder implements Seeder {
    private final VaccineDetailsRepository vaccineDetailsRepository;
    private final AutonomousCommunityRepository autonomousCommunityRepository;
    private final VaccineDosePeriodRepository vaccineDosePeriodRepository;
    private final VaccineRepository vaccineRepository;

    public VaccinesSeeder(VaccineDetailsRepository vaccineDetailsRepository, AutonomousCommunityRepository autonomousCommunityRepository, VaccineDosePeriodRepository vaccineDosePeriodRepository, VaccineRepository vaccineRepository) {
        this.vaccineDetailsRepository = vaccineDetailsRepository;
        this.autonomousCommunityRepository = autonomousCommunityRepository;
        this.vaccineDosePeriodRepository = vaccineDosePeriodRepository;
        this.vaccineRepository = vaccineRepository;
    }

    @Override
    public void seed() {
        if (vaccineDetailsRepository.findAll().isEmpty()) {
            List<VaccineDetailsEntity> vaccineDetailsList = Arrays.asList(
                    new VaccineDetailsEntity("Poliomielitis", "VPI"),
                    new VaccineDetailsEntity("Difteria, tétanos, tosferina", "DTPa"),
                    new VaccineDetailsEntity("Haemophilus influenzae b", "Hib"),
                    new VaccineDetailsEntity("Sarampión, rubeola, parotiditis", "TV"),
                    new VaccineDetailsEntity("Hepatitis B", "HB"),
                    new VaccineDetailsEntity("Enfermedad meningocócica", "MenB"),
                    new VaccineDetailsEntity("Enfermedad meningocócica", "MenC"),
                    new VaccineDetailsEntity("Enfermedad meningocócica", "MenACWY"),
                    new VaccineDetailsEntity("Varicela", "VVZ"),
                    new VaccineDetailsEntity("Virus del Papiloma Humano", "VPH"),
                    new VaccineDetailsEntity("Enfermedad neumocócica", "VNC"),
                    new VaccineDetailsEntity("Enfermedad neumocócica", "VN"),
                    new VaccineDetailsEntity("Virus Herpes Zóster", "HZ"),
                    new VaccineDetailsEntity("Gripe", "Gripe")
            );
            vaccineDetailsRepository.saveAll(vaccineDetailsList);

            // poliomielitis de Aragón
            List<Pair<Integer, Period>> poliomielitisAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 2, 0)),
                    Pair.of(2, Period.of(0, 4, 0)),
                    Pair.of(3, Period.of(0, 11, 0)),
                    Pair.of(4, Period.of(6, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(poliomielitisAragonDoses, "VPI", "Aragón", true);

            // dtpa de Aragón
            List<Pair<Integer, Period>> dtpaAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 2, 0)),
                    Pair.of(2, Period.of(0, 4, 0)),
                    Pair.of(3, Period.of(0, 11, 0)),
                    Pair.of(4, Period.of(6, 0, 0)),
                    Pair.of(5, Period.of(14, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(dtpaAragonDoses, "DTPa", "Aragón", true);

            // hib de Aragón
            List<Pair<Integer, Period>> hibAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 2, 0)),
                    Pair.of(2, Period.of(0, 4, 0)),
                    Pair.of(3, Period.of(0, 11, 0))
            );
            createAndSaveVaccinesAndDosePeriods(hibAragonDoses, "Hib", "Aragón", true);

            // tv de Aragón
            List<Pair<Integer, Period>> tvAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 12, 0)),
                    Pair.of(2, Period.of(3, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(tvAragonDoses, "TV", "Aragón", true);

            // hepatitisB de Aragón
            List<Pair<Integer, Period>> hepatitisBAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 0, 0)),
                    Pair.of(2, Period.of(0, 2, 0)),
                    Pair.of(3, Period.of(0, 4, 0)),
                    Pair.of(4, Period.of(0, 11, 0))
            );
            createAndSaveVaccinesAndDosePeriods(hepatitisBAragonDoses, "HB", "Aragón", true);

            // menB de Aragón
            List<Pair<Integer, Period>> menBAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 2, 0)),
                    Pair.of(2, Period.of(0, 4, 0)),
                    Pair.of(3, Period.of(0, 12, 0))
            );
            createAndSaveVaccinesAndDosePeriods(menBAragonDoses, "MenB", "Aragón", true);

            // menC de Aragón
            List<Pair<Integer, Period>> menCAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 4, 0)),
                    Pair.of(2, Period.of(0, 12, 0))
            );
            createAndSaveVaccinesAndDosePeriods(menCAragonDoses, "MenC", "Aragón", true);

            // menACWY de Aragón
            List<Pair<Integer, Period>> menACWYAragonDoses = Collections.singletonList(
                    Pair.of(1, Period.of(12, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(menACWYAragonDoses, "MenACWY", "Aragón", true);

            // vvz de Aragón
            List<Pair<Integer, Period>> vvzAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 15, 0)),
                    Pair.of(2, Period.of(3, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(vvzAragonDoses, "VVZ", "Aragón", true);

            //vph de Aragón
            List<Pair<Integer, Period>> vphAragonDoses = Collections.singletonList(
                    Pair.of(1, Period.of(12, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(vphAragonDoses, "VPH", "Aragón", true);

            // vnc de Aragón
            List<Pair<Integer, Period>> vncAragonDoses = Arrays.asList(
                    Pair.of(1, Period.of(0, 2, 0)),
                    Pair.of(2, Period.of(0, 4, 0)),
                    Pair.of(3, Period.of(0, 11, 0))
            );
            createAndSaveVaccinesAndDosePeriods(vncAragonDoses, "VNC", "Aragón", true);

            // hz de Aragón
            List<Pair<Integer, Period>> hzAragonDoses = Collections.singletonList(
                    Pair.of(1, Period.of(65, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(hzAragonDoses, "HZ", "Aragón", true);

            // gripe de Aragón
            List<Pair<Integer, Period>> gripeAragonDoses = Collections.singletonList(
                    Pair.of(0, Period.of(0, 0, 0))
            );
            createAndSaveVaccinesAndDosePeriods(gripeAragonDoses, "Gripe", "Aragón", false);
        }
    }

    private void createAndSaveVaccinesAndDosePeriods(List<Pair<Integer, Period>> doses, String vaccineShortName, String communityName, Boolean required) {
        VaccineDetailsEntity vaccineDetails = vaccineDetailsRepository.findByShortName(vaccineShortName);
        AutonomousCommunityEntity community = autonomousCommunityRepository.findByName(communityName);

        for (Pair<Integer, Period> dose : doses) {
            VaccineEntity vaccine = new VaccineEntity(dose.getFirst(), vaccineDetails);
            vaccineRepository.save(vaccine);

            VaccineDosePeriodEntity dosePeriod = new VaccineDosePeriodEntity(vaccine, community, dose.getSecond(), required);
            vaccine.getVaccineDosePeriods().add(dosePeriod);
            vaccineDosePeriodRepository.save(dosePeriod);
        }
    }
}
