package es.mdef.mipediatra.services.impl;

import es.mdef.mipediatra.assemblers.FileAssembler;
import es.mdef.mipediatra.entities.FileEntity;
import es.mdef.mipediatra.exceptions.EntityNotFoundException;
import es.mdef.mipediatra.exceptions.FileStorageException;
import es.mdef.mipediatra.models.PostModel;
import es.mdef.mipediatra.models.PutModel;
import es.mdef.mipediatra.models.file.FileListModel;
import es.mdef.mipediatra.models.file.FileModel;
import es.mdef.mipediatra.repositories.FileRepository;
import es.mdef.mipediatra.services.AssemblerService;
import es.mdef.mipediatra.services.CrudService;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@Service
public class FileStorageService implements CrudService<FileEntity, FileModel, PostModel, PutModel, FileListModel> {
    private final FileRepository fileRepository;
    private final FileAssembler assembler;

    public FileStorageService(FileRepository fileRepository, FileAssembler assembler) {
        this.fileRepository = fileRepository;
        this.assembler = assembler;
    }

    public FileEntity storeFile(MultipartFile file) {
        try {
            FileEntity fileEntity = new FileEntity();
            fileEntity.setName(file.getOriginalFilename());
            fileEntity.setType(file.getContentType());
            fileEntity.setData(file.getBytes());
            fileEntity.setSize(file.getSize());

            return fileRepository.save(fileEntity);
        } catch (IOException e) {
            throw new FileStorageException("Error al guardar " + file.getName() + ".", e);
        }
    }

    public FileModel getFile(Long fileId) {
        FileEntity file = fileRepository.findById(fileId)
                .orElseThrow(() -> new EntityNotFoundException(fileId, "archivo"));
        return assembler.toModel(file);
    }

    public FileEntity getFileEntity(Long fileId) {
        return fileRepository.findById(fileId)
                .orElseThrow(() -> new EntityNotFoundException(fileId, "archivo"));
    }

    public void deleteFile(FileEntity file) {
        fileRepository.delete(file);
    }

    @Override
    public String getEntityName() {
        return "file";
    }

    @Override
    public JpaRepository<FileEntity, Long> getRepository() {
        return fileRepository;
    }

    @Override
    public AssemblerService<FileEntity, FileModel, PostModel, PutModel, FileListModel> getAssembler() {
        return assembler;
    }
}
