package es.mdef.mipediatra.services.impl;

import es.mdef.mipediatra.assemblers.ReminderAssembler;
import es.mdef.mipediatra.entities.PatientEntity;
import es.mdef.mipediatra.entities.PersonEntity;
import es.mdef.mipediatra.entities.VaccineDosePeriodEntity;
import es.mdef.mipediatra.entities.VaccinesPersonsEntity;
import es.mdef.mipediatra.models.reminder.ReminderListModel;
import es.mdef.mipediatra.repositories.PatientRepository;
import es.mdef.mipediatra.repositories.VaccineDosePeriodRepository;
import es.mdef.mipediatra.repositories.VaccinesPersonsRepository;
import es.mdef.mipediatralib.interfaces.IReminder;
import es.mdef.mipediatralib.utils.ReminderManager;
import org.springframework.hateoas.CollectionModel;
import org.springframework.stereotype.Service;

import java.time.Period;
import java.util.ArrayList;
import java.util.List;

@Service
public class ReminderService {
    private final VaccineDosePeriodRepository vaccineDosePeriodRepository;
    private final PatientRepository patientRepository;
    private final VaccinesPersonsRepository vaccinesPersonsRepository;
    private final ReminderAssembler reminderAssembler;

    public ReminderService(VaccineDosePeriodRepository vaccineDosePeriodRepository, PatientRepository patientRepository, VaccinesPersonsRepository vaccinesPersonsRepository, ReminderAssembler reminderAssembler) {
        this.vaccineDosePeriodRepository = vaccineDosePeriodRepository;
        this.patientRepository = patientRepository;
        this.vaccinesPersonsRepository = vaccinesPersonsRepository;
        this.reminderAssembler = reminderAssembler;
    }

    public CollectionModel<ReminderListModel> readAll() {
        List<VaccineDosePeriodEntity> periods = vaccineDosePeriodRepository.findAll();
        List<VaccinesPersonsEntity> vaccinesPersons = vaccinesPersonsRepository.findAll();
        List<IReminder> reminders = new ArrayList<>();
        List<PatientEntity> patients = patientRepository.findAll();

        for (PatientEntity patient : patients) {
            if (patient instanceof PersonEntity person) {
                reminders.addAll(ReminderManager.getReminders(vaccinesPersons, periods, person, Period.of(0, 0, 30)));
            }
        }

        return reminderAssembler.toCollectionModel(reminders);
    }
}
